import googletrans
from tqdm import tqdm
import httpx
import pysrt


class Translator:
    def __init__(self, basename, src, dest):
        self.src, self.dest = src, dest
        self.subtitles = pysrt.open(f'{basename}.srt')
        self.progressbar = tqdm(total=len(self.subtitles.data), desc='Translating',
                                bar_format='{desc} --> {percentage:3.0f}%|{bar}|{n_fmt}/{total_fmt}')

        self.translator = None

    def translate_subtitle(self, subtitle):
        timeout = httpx.Timeout(10)
        self.translator = googletrans.Translator(service_urls=['translate.google.com', 'translate.google.co.kr'],
                                                 timeout=timeout)
        subtitle.text = self.translator.translate(subtitle.text, src=self.src, dest=self.dest).text
        self.progressbar.update(1)



from SubtitleTranslator.Translator import Translator
from SubtitleTranslator.constants import LANGUAGES
from multiprocessing.pool import ThreadPool
import click
import os


@click.group()
def cli():
    """Python Subtitle Translator as a command-line tool"""
    pass


@cli.command(help='Translate subtitle file.')
@click.argument('filename', type=click.Path(exists=True))
@click.option('--src', '-s', default='en', show_default=True,
              type=click.Choice(LANGUAGES.keys(), case_sensitive=False),
              help='The source language you want to translate.')
@click.option('--dest', '-d', default='fa', show_default=True,
              type=click.Choice(LANGUAGES.keys(), case_sensitive=False),
              help='The destination language you want to translate.')
@click.option('--processes', '-p', default=10, type=int, show_default=True,
              help='Number of processes.')
def translate(filename, src, dest, processes):
    basename, extension = os.path.splitext(filename)
    if extension == '.srt':
        translator = Translator(basename, src, dest)
        ThreadPool(processes).map(translator.translate_subtitle, translator.subtitles)
        translator.subtitles.save(f'{basename}.{dest}.srt')
    else:
        click.echo(f'Error : {filename} is not a valid subtitle srt file. (Valid Example is file.srt)')


@cli.command(help='Show available languages to translate.')
def show_lang():
    for key in LANGUAGES:
        click.echo(f'{key} = {LANGUAGES[key]}')



# -*- coding: utf-8 -*-
import io
import random
import shutil
import sys
from multiprocessing.pool import ThreadPool
import pathlib

import requests
from PIL import Image
import time


start = time.time()


def get_download_location():
    try:
        url_input = sys.argv[1]
    except IndexError:
        print('ERROR: Please provide the txt file\n$python image_downloader.py cats.txt')
    name = url_input.split('.')[0]
    pathlib.Path(name).mkdir(parents=True, exist_ok=True)
    return name


def get_urls():
    """
    Returns a list of urls by reading the txt file supplied as argument in terminal
    """
    try:
        url_input = sys.argv[1]
    except IndexError:
        print('ERROR: Please provide the txt file\n Example \n\n$python image_downloader.py dogs.txt \n\n')
        sys.exit()
    with open(url_input, 'r') as f:
        images_url = f.read().splitlines()

    print('{} Images detected'.format(len(images_url)))
    return images_url


def image_downloader(img_url: str):
    """
    Input:
    param: img_url  str (Image url)
    Tries to download the image url and use name provided in headers. Else it randomly picks a name
    """
    print(f'Downloading: {img_url}')
    res = requests.get(img_url, stream=True)
    count = 1
    while res.status_code != 200 and count <= 5:
        res = requests.get(img_url, stream=True)
        print(f'Retry: {count} {img_url}')
        count += 1
    # checking the type for image
    if 'image' not in res.headers.get("content-type", ''):
        print('ERROR: URL doesnot appear to be an image')
        return False
    # Trying to red image name from response headers
    try:
        image_name = str(img_url[(img_url.rfind('/')) + 1:])
        if '?' in image_name:
            image_name = image_name[:image_name.find('?')]
    except:
        image_name = str(random.randint(11111, 99999))+'.jpg'

    i = Image.open(io.BytesIO(res.content))
    download_location = get_download_location()
    i.save(download_location + '/'+image_name)
    return f'Download complete: {img_url}'


def run_downloader(process: int, images_url: list):
    """
    Inputs:
        process: (int) number of process to run
        images_url:(list) list of images url
    """
    print(f'MESSAGE: Running {process} process')
    results = ThreadPool(process).imap_unordered(image_downloader, images_url)
    for r in results:
        print(r)


try:
    num_process = int(sys.argv[2])
except:
    num_process = 10

images_url = get_urls()
run_downloader(num_process, images_url)


end = time.time()
print('Time taken to download {}'.format(len(get_urls())))
print(end - start)